@@ -21,7 +21,7 @@ abstract class ScriptTest extends DirectTest {
   def testmain = "TestMain"
   override def extraSettings = s"-usejavacp -Xscript $testmain"
   def scriptPath = testPath changeExtension "script"
-  def code = scriptPath.toFile.slurp
+  def code = scriptPath.toFile.slurp()
   def argv = Seq.empty[String]
   def show() = {
     compile()
