@@ -124,31 +124,21 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
   }
 
   /** Evaluate an action body and judge whether it passed. */
-  def nextTestAction[T](body: => T)(eval: PartialFunction[T, TestState]): TestState = eval.applyOrElse(body, (_: T) => genPass)
+  def nextTestAction[T](body: => T)(eval: PartialFunction[T, TestState]): TestState = eval.applyOrElse(body, (_: T) => genPass())
   /** If the action does not result in true, fail the action. */
   def nextTestActionExpectTrue(reason: String, body: => Boolean): TestState = nextTestAction(body) { case false => genFail(reason) }
   /** Fail the action. */
   def nextTestActionFailing(reason: String): TestState = nextTestActionExpectTrue(reason, false)
 
-  private def assembleTestCommand(outDir: File, logFile: File): List[String] = {
-    // check whether there is a ".javaopts" file
-    val argsFile  = testFile changeExtension "javaopts"
-    val javaopts = readOptionsFile(argsFile)
+  private def assembleTestCommand(outDir: File, javaopts: List[String]): List[String] = {
     if (javaopts.nonEmpty)
-      suiteRunner.verbose(s"Found javaopts file '$argsFile', using options: '${javaopts.mkString(",")}'")
+      suiteRunner.verbose(s"Using java options: '${javaopts.mkString(",")}'")
 
-    // Note! As this currently functions, suiteRunner.javaOpts must precede argString
-    // because when an option is repeated to java only the last one wins.
-    // That means until now all the .javaopts files were being ignored because
-    // they all attempt to change options which are also defined in
-    // partest.java_opts, leading to debug output like:
-    //
-    // debug: Found javaopts file 'files/shootout/message.scala-2.javaopts', using options: '-Xss32k'
-    // debug: java -Xss32k -Xss2m -Xms256M -Xmx1024M -classpath [...]
     val propertyOpts = propertyOptions(fork = true).map { case (k, v) => s"-D$k=$v" }
 
     val classpath = joinPaths(extraClasspath ++ testClassPath)
 
+    // `javaopts` last; for repeated arguments, the last one wins
     javaCmdPath +: (
       (suiteRunner.javaOpts.split(' ') ++ extraJavaOptions ++ javaopts).filter(_ != "").toList ++ Seq(
         "-classpath",
@@ -208,15 +198,15 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
             case Failure(e) => outFile.appendAll(stackTraceString(e)) ; return -1
             case Success(v) => v
           }
-        try p.exitValue
+        try p.exitValue()
         catch {
           case e: InterruptedException =>
             suiteRunner.verbose(s"Interrupted waiting for command to finish (${args mkString " "})")
-            p.destroy
+            p.destroy()
             nonzero
           case t: Throwable =>
             suiteRunner.verbose(s"Exception waiting for command to finish: $t (${args mkString " "})")
-            p.destroy
+            p.destroy()
             throw t
         }
       }
@@ -224,8 +214,8 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
     }
   }
 
-  private def execTest(outDir: File, logFile: File): TestState = {
-    val cmd = assembleTestCommand(outDir, logFile)
+  private def execTest(outDir: File, logFile: File, javaopts: List[String]): TestState = {
+    val cmd = assembleTestCommand(outDir, javaopts)
 
     pushTranscript((cmd mkString s" \\$EOL  ") + " > " + logFile.getName)
     nextTestAction(runCommand(cmd, logFile)) {
@@ -272,7 +262,7 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
         case Left((status, throwable)) if status != 0 =>
           genFail("non-zero exit code")
         case _ =>
-          genPass
+          genPass()
       }
     }
   }
@@ -412,7 +402,7 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
     normalizeLog()
     pushTranscript(s"diff $checkFile $logFile")
     currentDiff match {
-      case "" => genPass
+      case "" => genPass()
       case diff if config.optUpdateCheck =>
         suiteRunner.verbose("Updating checkfile " + checkFile)
         checkFile.writeAll(logFile.fileContents)
@@ -514,9 +504,35 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
     def description = mkScalacString()
     lazy val result = { pushTranscript(description) ; attemptCompile(fs) }
   }
+  case class SkipRound(fs: List[File], state: TestState) extends CompileRound {
+    def description: String = state.status
+    lazy val result = { pushTranscript(description); state }
+  }
+
+  def compilationRounds(file: File): List[CompileRound] = {
+    import scala.util.Properties.javaSpecVersion
+    val Range = """(\d+)(?:(\+)|(?: *\- *(\d+)))?""".r
+    lazy val currentJavaVersion = javaSpecVersion.stripPrefix("1.").toInt
+    val allFiles = sources(file)
+    val skipStates = toolArgsFor(allFiles)("javaVersion", split = false).flatMap({
+      case v @ Range(from, plus, to) =>
+        val ok =
+          if (plus == null)
+            if (to == null) currentJavaVersion == from.toInt
+            else from.toInt <= currentJavaVersion && currentJavaVersion <= to.toInt
+          else
+            currentJavaVersion >= from.toInt
+        if (ok) None
+        else Some(genSkip(s"skipped on Java $javaSpecVersion, only running on $v"))
+      case v =>
+        Some(genFail(s"invalid javaVersion range in test comment: $v"))
+    })
+    skipStates.headOption match {
+      case Some(state) => List(SkipRound(List(file), state))
+      case _ => groupedFiles(allFiles).flatMap(mixedCompileGroup)
+    }
+  }
 
-  def compilationRounds(file: File): List[CompileRound] =
-    groupedFiles(sources(file)).map(mixedCompileGroup).flatten
   def mixedCompileGroup(allFiles: List[File]): List[CompileRound] = {
     val (scalaFiles, javaFiles) = allFiles partition (_.isScala)
     val round1                  = if (scalaFiles.isEmpty) None else Some(ScalaAndJava(allFiles))
@@ -533,17 +549,18 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
     // pass if it checks and didn't crash the compiler
     // or, OK, we'll let you crash the compiler with a FatalError if you supply a check file
     def checked(r: CompileRound) = r.result match {
+      case s: Skip => s
       case crash @ Crash(_, t, _) if !checkFile.canRead || !t.isInstanceOf[FatalError] => crash
-      case dnc @ _ => diffIsOk
+      case _ => diffIsOk
     }
 
-    compilationRounds(testFile).find(!_.result.isOk).map(checked).getOrElse(genFail("expected compilation failure"))
+    compilationRounds(testFile).find(r => !r.result.isOk || r.result.isSkipped).map(checked).getOrElse(genFail("expected compilation failure"))
   }
 
   // run compilation until failure, evaluate `andAlso` on success
-  def runTestCommon(andAlso: => TestState = genPass): TestState = runInContext {
+  def runTestCommon(andAlso: => TestState = genPass()): TestState = runInContext {
     // DirectCompiler already says compilation failed
-    val res = compilationRounds(testFile).find(!_.result.isOk).map(_.result).getOrElse(genPass)
+    val res = compilationRounds(testFile).find(r => !r.result.isOk || r.result.isSkipped).map(_.result).getOrElse(genPass())
     res andAlso andAlso
   }
 
@@ -609,7 +626,7 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
     def loop(): TestState = {
       logWriter.print(prompt)
       resReader.readLine() match {
-        case null | ""  => logWriter.close() ; genPass
+        case null | ""  => logWriter.close() ; genPass()
         case line       => resCompile(line) andAlso loop()
       }
     }
@@ -635,14 +652,13 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
       case k if k.endsWith("-neg") => runNegTest()
       case _              => runRunTest()
     }
-    (state, stopwatch.stop)
+    (state, stopwatch.stop())
   }
 
   private def runRunTest(): TestState = {
-    val argsFile = testFile changeExtension "javaopts"
-    val javaopts = readOptionsFile(argsFile)
+    val javaopts = toolArgs("java")
     val execInProcess = PartestDefaults.execInProcess && javaopts.isEmpty && !Set("specialized", "instrumented").contains(testFile.getParentFile.getName)
-    def exec() = if (execInProcess) execTestInProcess(outDir, logFile) else execTest(outDir, logFile)
+    def exec() = if (execInProcess) execTestInProcess(outDir, logFile) else execTest(outDir, logFile, javaopts)
     def noexec() = genSkip("no-exec: tests compiled but not run")
     runTestCommon(if (suiteRunner.config.optNoExec) noexec() else exec().andAlso(diffIsOk))
   }
@@ -669,7 +685,7 @@ class Runner(val testInfo: TestInfo, val suiteRunner: AbstractRunner) {
     val cmdFile = if (isWin) testFile changeExtension "bat" else testFile
     val succeeded = (((s"$cmdFile $args" #> logFile).!) == 0)
 
-    val result = if (succeeded) genPass else genFail(s"script $cmdFile failed to run")
+    val result = if (succeeded) genPass() else genFail(s"script $cmdFile failed to run")
 
     result andAlso diffIsOk
   }
